import AssignmentProperty from "./elements/assignment-property.js";
import Property from "./elements/property.js";
import SpreadElement from "./elements/spread-element.js";
import ExpressionVisitor from "./expression-visitor.js";
import ArrayExpression from "./expressions/array-expression.js";
import ArrowFunctionExpression from "./expressions/arrow-function-expression.js";
import AssignmentExpression from "./expressions/assignment-expression.js";
import BinaryExpression from "./expressions/binary-expression.js";
import CallExpression from "./expressions/call-expression.js";
import ChainExpression from "./expressions/chain-expression.js";
import ConditionalExpression from "./expressions/conditional-expression.js";
import LogicalExpression from "./expressions/logical-expression.js";
import MemberExpression from "./expressions/member-expression.js";
import NewExpression from "./expressions/new-expression.js";
import ObjectExpression from "./expressions/object-expression.js";
import ParenthesizedExpression from "./expressions/parenthesized-expression.js";
import SequenceExpression from "./expressions/sequence-expression.js";
import TaggedTemplateExpression from "./expressions/tagged-template-expression.js";
import TemplateLiteral from "./expressions/template-literal.js";
import UnaryExpression from "./expressions/unary-expression.js";
import UpdateExpression from "./expressions/update-expression.js";
import ArrayPattern from "./patterns/array-pattern.js";
import AssignmentPattern from "./patterns/assignment-pattern.js";
import ObjectPattern from "./patterns/object-pattern.js";
import RestElement from "./patterns/rest-element.js";
export default class ExpressionRewriterVisitor extends ExpressionVisitor {
    visitArrayExpression(node) {
        return new ArrayExpression(node.elements.map(x => x ? this.visit(x) : x));
    }
    visitArrayPattern(node) {
        return new ArrayPattern(node.elements.map(x => x ? this.visit(x) : x));
    }
    visitArrowFunctionExpression(node) {
        return new ArrowFunctionExpression(node.parameters.map(x => this.visit(x)), this.visit(node.body));
    }
    visitAssignmentExpression(node) {
        return new AssignmentExpression(this.visit(node.left), this.visit(node.right), node.operator);
    }
    visitAssignmentPattern(node) {
        return new AssignmentPattern(this.visit(node.left), this.visit(node.right));
    }
    visitAssignmentProperty(node) {
        return new AssignmentProperty(this.visit(node.key), this.visit(node.value), node.computed, node.shorthand);
    }
    visitBinaryExpression(node) {
        return new BinaryExpression(this.visit(node.left), this.visit(node.right), node.operator);
    }
    visitCallExpression(node) {
        return new CallExpression(this.visit(node.callee), node.arguments.map(x => this.visit(x)));
    }
    visitChainExpression(node) {
        return new ChainExpression(this.visit(node.expression));
    }
    visitConditionalExpression(node) {
        return new ConditionalExpression(this.visit(node.test), this.visit(node.alternate), this.visit(node.consequent));
    }
    visitLogicalExpression(node) {
        return new LogicalExpression(this.visit(node.left), this.visit(node.right), node.operator);
    }
    visitMemberExpression(node) {
        return new MemberExpression(this.visit(node.object), this.visit(node.property), node.computed, node.optional);
    }
    visitNewExpression(node) {
        return new NewExpression(this.visit(node.callee), node.arguments.map(x => this.visit(x)));
    }
    visitObjectExpression(node) {
        return new ObjectExpression(node.properties.map(x => this.visit(x)));
    }
    visitObjectPattern(node) {
        return new ObjectPattern(node.properties.map(x => this.visit(x)));
    }
    visitParenthesizedExpression(node) {
        return new ParenthesizedExpression(this.visit(node.argument));
    }
    visitProperty(node) {
        return new Property(this.visit(node.key), this.visit(node.value), node.computed, node.shorthand);
    }
    visitRestElement(node) {
        return new RestElement(this.visit(node.argument));
    }
    visitSequenceExpression(node) {
        return new SequenceExpression(node.expressions.map(x => this.visit(x)));
    }
    visitSpreadExpression(node) {
        return new SpreadElement(this.visit(node.argument));
    }
    visitTaggedTemplateExpression(node) {
        return new TaggedTemplateExpression(this.visit(node.callee), this.visit(node.quasi));
    }
    visitTemplateLiteral(node) {
        return new TemplateLiteral(node.quasis.map(x => this.visit(x)), node.expressions.map(x => this.visit(x)));
    }
    visitUnaryExpression(node) {
        return new UnaryExpression(this.visit(node.argument), node.operator);
    }
    visitUpdateExpression(node) {
        return new UpdateExpression(this.visit(node.argument), node.operator, node.prefix);
    }
}
