import type AssignmentProperty from "./elements/assignment-property.js";
import type Property from "./elements/property.js";
import type SpreadElement from "./elements/spread-element.js";
import type TemplateElement from "./elements/template-element.js";
import type ArrayExpression from "./expressions/array-expression.js";
import type ArrowFunctionExpression from "./expressions/arrow-function-expression.js";
import type AssignmentExpression from "./expressions/assignment-expression.js";
import type BinaryExpression from "./expressions/binary-expression.js";
import type CallExpression from "./expressions/call-expression.js";
import type ChainExpression from "./expressions/chain-expression.js";
import type ConditionalExpression from "./expressions/conditional-expression.js";
import type Identifier from "./expressions/identifier.js";
import type Literal from "./expressions/literal.js";
import type LogicalExpression from "./expressions/logical-expression.js";
import type MemberExpression from "./expressions/member-expression.js";
import type NewExpression from "./expressions/new-expression.js";
import type ObjectExpression from "./expressions/object-expression.js";
import type ParenthesizedExpression from "./expressions/parenthesized-expression.js";
import type SequenceExpression from "./expressions/sequence-expression.js";
import type TaggedTemplateExpression from "./expressions/tagged-template-expression.js";
import type TemplateLiteral from "./expressions/template-literal.js";
import type ThisExpression from "./expressions/this-expression.js";
import type UnaryExpression from "./expressions/unary-expression.js";
import type UpdateExpression from "./expressions/update-expression.js";
import type INode from "./interfaces/node.js";
import type ArrayPattern from "./patterns/array-pattern.js";
import type AssignmentPattern from "./patterns/assignment-pattern.js";
import type ObjectPattern from "./patterns/object-pattern.js";
import type RestElement from "./patterns/rest-element.js";
export default abstract class ExpressionVisitor {
    protected visit(node: INode): INode;
    protected visitArrayExpression(node: ArrayExpression): INode;
    protected visitArrayPattern(node: ArrayPattern): INode;
    protected visitArrowFunctionExpression(node: ArrowFunctionExpression): INode;
    protected visitAssignmentExpression(node: AssignmentExpression): INode;
    protected visitAssignmentProperty(node: AssignmentProperty): INode;
    protected visitAssignmentPattern(node: AssignmentPattern): INode;
    protected visitBinaryExpression(node: BinaryExpression): INode;
    protected visitCallExpression(node: CallExpression): INode;
    protected visitChainExpression(node: ChainExpression): INode;
    protected visitConditionalExpression(node: ConditionalExpression): INode;
    protected visitIdentifier(node: Identifier): INode;
    protected visitLiteral(node: Literal): INode;
    protected visitLogicalExpression(node: LogicalExpression): INode;
    protected visitMemberExpression(node: MemberExpression): INode;
    protected visitNewExpression(node: NewExpression): INode;
    protected visitObjectExpression(node: ObjectExpression): INode;
    protected visitObjectPattern(node: ObjectPattern): INode;
    protected visitParenthesizedExpression(node: ParenthesizedExpression): INode;
    protected visitProperty(node: Property): INode;
    protected visitRestElement(node: RestElement): INode;
    protected visitSequenceExpression(node: SequenceExpression): INode;
    protected visitSpreadExpression(node: SpreadElement): INode;
    protected visitTaggedTemplateExpression(node: TaggedTemplateExpression): INode;
    protected visitTemplateLiteral(node: TemplateLiteral): INode;
    protected visitTemplateElement(node: TemplateElement): INode;
    protected visitThisExpression(node: ThisExpression): INode;
    protected visitUnaryExpression(node: UnaryExpression): INode;
    protected visitUpdateExpression(node: UpdateExpression): INode;
}
