import TypeGuard from "./type-guard.js";
export default class ExpressionVisitor {
    visit(node) {
        if (TypeGuard.isArrayExpression(node)) {
            return this.visitArrayExpression(node);
        }
        else if (TypeGuard.isArrayPattern(node)) {
            return this.visitArrayPattern(node);
        }
        else if (TypeGuard.isArrowFunctionExpression(node)) {
            return this.visitArrowFunctionExpression(node);
        }
        else if (TypeGuard.isAssignmentExpression(node)) {
            return this.visitAssignmentExpression(node);
        }
        else if (TypeGuard.isAssignmentProperty(node)) {
            return this.visitAssignmentProperty(node);
        }
        else if (TypeGuard.isAssignmentPattern(node)) {
            return this.visitAssignmentPattern(node);
        }
        else if (TypeGuard.isBinaryExpression(node)) {
            return this.visitBinaryExpression(node);
        }
        else if (TypeGuard.isCallExpression(node)) {
            return this.visitCallExpression(node);
        }
        else if (TypeGuard.isChainExpression(node)) {
            return this.visitChainExpression(node);
        }
        else if (TypeGuard.isConditionalExpression(node)) {
            return this.visitConditionalExpression(node);
        }
        else if (TypeGuard.isIdentifier(node)) {
            return this.visitIdentifier(node);
        }
        else if (TypeGuard.isLiteral(node)) {
            return this.visitLiteral(node);
        }
        else if (TypeGuard.isLogicalExpression(node)) {
            return this.visitLogicalExpression(node);
        }
        else if (TypeGuard.isMemberExpression(node)) {
            return this.visitMemberExpression(node);
        }
        else if (TypeGuard.isNewExpression(node)) {
            return this.visitNewExpression(node);
        }
        else if (TypeGuard.isObjectExpression(node)) {
            return this.visitObjectExpression(node);
        }
        else if (TypeGuard.isObjectPattern(node)) {
            return this.visitObjectPattern(node);
        }
        else if (TypeGuard.isParenthesizedExpression(node)) {
            return this.visitParenthesizedExpression(node);
        }
        else if (TypeGuard.isProperty(node)) {
            return this.visitProperty(node);
        }
        else if (TypeGuard.isRestElement(node)) {
            return this.visitRestElement(node);
        }
        else if (TypeGuard.isSequenceExpression(node)) {
            return this.visitSequenceExpression(node);
        }
        else if (TypeGuard.isSpreadElement(node)) {
            return this.visitSpreadExpression(node);
        }
        else if (TypeGuard.isTaggedTemplateExpression(node)) {
            return this.visitTaggedTemplateExpression(node);
        }
        else if (TypeGuard.isTemplateLiteral(node)) {
            return this.visitTemplateLiteral(node);
        }
        else if (TypeGuard.isTemplateElement(node)) {
            return this.visitTemplateElement(node);
        }
        else if (TypeGuard.isThisExpression(node)) {
            return this.visitThisExpression(node);
        }
        else if (TypeGuard.isUpdateExpression(node)) {
            return this.visitUpdateExpression(node);
        }
        else if (TypeGuard.isUnaryExpression(node)) {
            return this.visitUnaryExpression(node);
        } /* c8 ignore next 3 */
        return node;
    }
    visitArrayExpression(node) {
        for (const element of node.elements) {
            if (element) {
                this.visit(element);
            }
        }
        return node;
    }
    visitArrayPattern(node) {
        for (const element of node.elements) {
            if (element) {
                this.visit(element);
            }
        }
        return node;
    }
    visitArrowFunctionExpression(node) {
        for (const parameter of node.parameters) {
            this.visit(parameter);
        }
        this.visit(node.body);
        return node;
    }
    visitAssignmentExpression(node) {
        this.visit(node.left);
        this.visit(node.right);
        return node;
    }
    visitAssignmentProperty(node) {
        this.visit(node.key);
        this.visit(node.value);
        return node;
    }
    visitAssignmentPattern(node) {
        this.visit(node.left);
        this.visit(node.right);
        return node;
    }
    visitBinaryExpression(node) {
        this.visit(node.left);
        this.visit(node.right);
        return node;
    }
    visitCallExpression(node) {
        this.visit(node.callee);
        for (const arg of node.arguments) {
            this.visit(arg);
        }
        return node;
    }
    visitChainExpression(node) {
        this.visit(node.expression);
        return node;
    }
    visitConditionalExpression(node) {
        this.visit(node.test);
        this.visit(node.alternate);
        this.visit(node.consequent);
        return node;
    }
    visitIdentifier(node) {
        return node;
    }
    visitLiteral(node) {
        return node;
    }
    visitLogicalExpression(node) {
        this.visit(node.left);
        this.visit(node.right);
        return node;
    }
    visitMemberExpression(node) {
        this.visit(node.object);
        this.visit(node.property);
        return node;
    }
    visitNewExpression(node) {
        this.visit(node.callee);
        for (const arg of node.arguments) {
            this.visit(arg);
        }
        return node;
    }
    visitObjectExpression(node) {
        for (const entry of node.properties) {
            this.visit(entry);
        }
        return node;
    }
    visitObjectPattern(node) {
        for (const entry of node.properties) {
            this.visit(entry);
        }
        return node;
    }
    visitParenthesizedExpression(node) {
        this.visit(node.argument);
        return node;
    }
    visitProperty(node) {
        this.visit(node.key);
        this.visit(node.value);
        return node;
    }
    visitRestElement(node) {
        this.visit(node.argument);
        return node;
    }
    visitSequenceExpression(node) {
        for (const expression of node.expressions) {
            this.visit(expression);
        }
        return node;
    }
    visitSpreadExpression(node) {
        this.visit(node.argument);
        return node;
    }
    visitTaggedTemplateExpression(node) {
        this.visit(node.callee);
        this.visit(node.quasi);
        return node;
    }
    visitTemplateLiteral(node) {
        for (const quasi of node.quasis) {
            this.visit(quasi);
        }
        for (const expression of node.expressions) {
            this.visit(expression);
        }
        return node;
    }
    visitTemplateElement(node) {
        return node;
    }
    visitThisExpression(node) {
        return node;
    }
    visitUnaryExpression(node) {
        this.visit(node.argument);
        return node;
    }
    visitUpdateExpression(node) {
        this.visit(node.argument);
        return node;
    }
}
