import NodeType from "../node-type.js";
import TypeGuard from "../type-guard.js";
export default class ArrayExpression {
    _elements;
    get elements() {
        return this._elements;
    }
    /* c8 ignore next 4 */
    set elements(value) {
        this._elements = value;
    }
    get type() {
        return NodeType.ArrayExpression;
    }
    constructor(elements) {
        this._elements = elements;
    }
    clone() {
        return new ArrayExpression(this.elements.map(x => x?.clone() ?? null));
    }
    evaluate(scope) {
        const evaluation = [];
        for (const element of this.elements) {
            if (element) {
                if (TypeGuard.isSpreadElement(element)) {
                    evaluation.push(...element.argument.evaluate(scope));
                }
                else {
                    evaluation.push(element.evaluate(scope));
                }
            }
            else {
                evaluation.push(undefined);
            }
        }
        return evaluation;
    }
    toString() {
        return `[${this.elements.map(x => (x ?? "undefined").toString()).join(", ")}]`;
    }
}
