import { proxyFrom } from "@surface/core";
import NodeType from "../node-type.js";
import TypeGuard from "../type-guard.js";
export default class ArrowFunctionExpression {
    _body;
    get body() {
        return this._body;
    }
    /* c8 ignore next 4 */
    set body(value) {
        this._body = value;
    }
    _parameters;
    get parameters() {
        return this._parameters;
    }
    /* c8 ignore next 4 */
    set parameters(value) {
        this._parameters = value;
    }
    get type() {
        return NodeType.ArrowFunctionExpression;
    }
    constructor(parameters, body) {
        this._parameters = parameters;
        this._body = body;
    }
    resolveParameters(scope, $arguments) {
        const currentScope = {};
        let index = 0;
        for (const parameter of this.parameters) {
            if (TypeGuard.isRestElement(parameter)) {
                Object.assign(currentScope, parameter.evaluate(scope, $arguments.slice(index)));
            }
            else {
                Object.assign(currentScope, parameter.evaluate(scope, $arguments[index]));
            }
            index++;
        }
        return currentScope;
    }
    clone() {
        return new ArrowFunctionExpression(this.parameters.map(x => x.clone()), this.body.clone());
    }
    evaluate(scope) {
        const fn = (...args) => this.body.evaluate(proxyFrom(this.resolveParameters(scope, args), scope));
        fn.toString = () => this.toString();
        return fn;
    }
    toString() {
        return `(${this.parameters.map(x => x.toString()).join(", ")}) => ${this.body}`;
    }
}
