import NodeType from "../node-type.js";
import TypeGuard from "../type-guard.js";
const assignmentOperations = {
    "%=": (target, key, value) => target[key] %= value,
    "&&=": (target, key, value) => target[key] &&= value,
    "&=": (target, key, value) => target[key] &= value,
    "**=": (target, key, value) => target[key] **= value,
    "*=": (target, key, value) => target[key] *= value,
    "+=": (target, key, value) => target[key] += value,
    "-=": (target, key, value) => target[key] -= value,
    "/=": (target, key, value) => target[key] /= value,
    "<<=": (target, key, value) => target[key] <<= value,
    "=": (target, key, value) => target[key] = value,
    ">>=": (target, key, value) => target[key] >>= value,
    ">>>=": (target, key, value) => target[key] >>>= value,
    "??=": (target, key, value) => target[key] ??= value,
    "^=": (target, key, value) => target[key] ^= value,
    "|=": (target, key, value) => target[key] |= value,
    "||=": (target, key, value) => target[key] ||= value,
};
export default class AssignmentExpression {
    operation;
    _left;
    get left() {
        return this._left;
    }
    /* c8 ignore next 4 */
    set left(value) {
        this._left = value;
    }
    _operator;
    get operator() {
        return this._operator;
    }
    /* c8 ignore next 4 */
    set operator(value) {
        this._operator = value;
    }
    _right;
    get right() {
        return this._right;
    }
    /* c8 ignore next 4 */
    set right(value) {
        this._right = value;
    }
    get type() {
        return NodeType.AssignmentExpression;
    }
    constructor(left, right, operator) {
        this._left = left;
        this._right = right;
        this._operator = operator;
        this.operation = assignmentOperations[operator];
    }
    clone() {
        return new AssignmentExpression(this.left.clone(), this.right.clone(), this.operator);
    }
    evaluate(scope) {
        if (TypeGuard.isIdentifier(this.left)) {
            return this.operation(scope, this.left.name, this.right.evaluate(scope));
        }
        const object = this.left.object.evaluate(scope);
        const property = TypeGuard.isIdentifier(this.left.property) && !this.left.computed ? this.left.property.name : this.left.property.evaluate(scope);
        return this.operation(object, property, this.right.evaluate(scope));
    }
    toString() {
        return `${this.left} ${this.operator} ${this.right}`;
    }
}
