import { format, hasValue } from "@surface/core";
import { getThisArg } from "../common.js";
import Messages from "../messages.js";
import NodeType from "../node-type.js";
import TypeGuard from "../type-guard.js";
export default class CallExpression {
    _arguments;
    get arguments() {
        return this._arguments;
    }
    /* c8 ignore next 4 */
    set arguments(value) {
        this._arguments = value;
    }
    _callee;
    get callee() {
        return this._callee;
    }
    /* c8 ignore next 4 */
    set callee(value) {
        this._callee = value;
    }
    _optional;
    get optional() {
        return this._optional;
    }
    /* c8 ignore next 4 */
    set optional(value) {
        this._optional = value;
    }
    get type() {
        return NodeType.CallExpression;
    }
    constructor(callee, $arguments = [], optional = false) {
        this._arguments = $arguments;
        this._callee = callee;
        this._optional = optional;
    }
    clone() {
        return new CallExpression(this.callee.clone(), this.arguments.map(x => x.clone()), this.optional);
    }
    evaluate(scope) {
        const [thisArg, fn] = getThisArg(this.callee, scope);
        if (this.optional && !hasValue(fn)) {
            return undefined;
        }
        if (!fn) {
            throw new ReferenceError(format(Messages.identifierIsNotDefined, { identifier: this.callee.toString() }));
        }
        else if (typeof fn != "function") {
            throw new TypeError(format(Messages.identifierIsNotAFunction, { identifier: this.callee.toString() }));
        }
        const $arguments = [];
        for (const argument of this.arguments) {
            if (TypeGuard.isSpreadElement(argument)) {
                $arguments.push(...argument.argument.evaluate(scope));
            }
            else {
                $arguments.push(argument.evaluate(scope));
            }
        }
        return fn.apply(thisArg, $arguments);
    }
    toString() {
        // eslint-disable-next-line max-len
        return `${[NodeType.BinaryExpression, NodeType.ConditionalExpression, NodeType.ArrowFunctionExpression].includes(this.callee.type) ? `(${this.callee})` : this.callee}${this.optional ? "?." : ""}(${this.arguments.map(x => x.toString()).join(", ")})`;
    }
}
