import { hasValue } from "@surface/core";
import NodeType from "../node-type.js";
import TypeGuard from "../type-guard.js";
export default class MemberExpression {
    _computed;
    get computed() {
        return this._computed;
    }
    /* c8 ignore next 4 */
    set computed(value) {
        this._computed = value;
    }
    _property;
    get property() {
        return this._property;
    }
    /* c8 ignore next 4 */
    set property(value) {
        this._property = value;
    }
    _object;
    get object() {
        return this._object;
    }
    /* c8 ignore next 4 */
    set object(value) {
        this._object = value;
    }
    _optional;
    get optional() {
        return this._optional;
    }
    /* c8 ignore next 4 */
    set optional(value) {
        this._optional = value;
    }
    get type() {
        return NodeType.MemberExpression;
    }
    constructor(object, property, computed, optional) {
        this._object = object;
        this._property = property;
        this._computed = computed;
        this._optional = optional;
    }
    clone() {
        return new MemberExpression(this.object.clone(), this.property.clone(), this.computed, this.optional);
    }
    evaluate(scope) {
        const object = this.object.evaluate(scope);
        const key = TypeGuard.isIdentifier(this.property) && !this.computed ? this.property.name : `${this.property.evaluate(scope)}`;
        if (this.optional) {
            return hasValue(object) ? object[key] : undefined;
        }
        return object[key];
    }
    toString() {
        return `${this.object}${TypeGuard.isIdentifier(this.property) && !this.computed ? `${this.optional ? "?" : ""}.${this.property.name}` : `${this.optional ? "?." : ""}[${this.property}]`}`;
    }
}
