import { format } from "@surface/core";
import Messages from "../messages.js";
import NodeType from "../node-type.js";
import TypeGuard from "../type-guard.js";
export default class NewExpression {
    _arguments;
    get arguments() {
        return this._arguments;
    }
    /* c8 ignore next 4 */
    set arguments(value) {
        this._arguments = value;
    }
    _callee;
    get callee() {
        return this._callee;
    }
    /* c8 ignore next 4 */
    set callee(value) {
        this._callee = value;
    }
    get type() {
        return NodeType.NewExpression;
    }
    constructor(callee, $arguments = []) {
        this._callee = callee;
        this._arguments = $arguments;
    }
    clone() {
        return new NewExpression(this.callee.clone(), this.arguments.map(x => x.clone()));
    }
    evaluate(scope) {
        const fn = this.callee.evaluate(scope);
        if (!fn) {
            throw new ReferenceError(format(Messages.identifierIsNotDefined, { identifier: this.callee.toString() }));
        }
        else if (typeof fn != "function") {
            throw new TypeError(format(Messages.identifierIsNotAConstructor, { identifier: this.callee.toString() }));
        }
        const $arguments = [];
        for (const argument of this.arguments) {
            if (TypeGuard.isSpreadElement(argument)) {
                $arguments.push(...argument.argument.evaluate(scope));
            }
            else {
                $arguments.push(argument.evaluate(scope));
            }
        }
        return Reflect.construct(fn, $arguments, fn);
    }
    toString() {
        return `new ${this.callee}(${this.arguments.map(x => x.toString()).join(", ")})`;
    }
}
