import NodeType from "../node-type.js";
import TypeGuard from "../type-guard.js";
export default class ObjectExpression {
    _properties;
    get properties() {
        return this._properties;
    }
    /* c8 ignore next 4 */
    set properties(value) {
        this._properties = value;
    }
    get type() {
        return NodeType.ObjectExpression;
    }
    constructor(properties = []) {
        this._properties = properties;
    }
    clone() {
        return new ObjectExpression(this.properties.map(x => x.clone()));
    }
    evaluate(scope) {
        const evaluation = {};
        for (const property of this.properties) {
            if (TypeGuard.isProperty(property)) {
                const key = TypeGuard.isIdentifier(property.key) && !property.computed ? property.key.name : property.key.evaluate(scope);
                evaluation[key] = property.value.evaluate(scope);
            }
            else {
                Object.assign(evaluation, property.argument.evaluate(scope));
            }
        }
        return evaluation;
    }
    toString() {
        return this.properties.length > 0 ? `{ ${this.properties.map(x => x.toString()).join(", ")} }` : "{ }";
    }
}
