import { format } from "@surface/core";
import { getThisArg } from "../common.js";
import Messages from "../messages.js";
import NodeType from "../node-type.js";
export default class TaggedTemplateExpression {
    _callee;
    get callee() {
        return this._callee;
    }
    /* c8 ignore next 4 */
    set callee(value) {
        this._callee = value;
    }
    _quasi;
    get quasi() {
        return this._quasi;
    }
    /* c8 ignore next 4 */
    set quasi(value) {
        this._quasi = value;
    }
    get type() {
        return NodeType.TaggedTemplateExpression;
    }
    constructor(callee, quasi) {
        this._callee = callee,
            this._quasi = quasi;
    }
    clone() {
        return new TaggedTemplateExpression(this.callee.clone(), this.quasi.clone());
    }
    evaluate(scope) {
        const [thisArg, fn] = getThisArg(this.callee, scope);
        if (!fn) {
            throw new ReferenceError(format(Messages.identifierIsNotDefined, { identifier: this.callee.toString() }));
        }
        else if (typeof fn != "function") {
            throw new TypeError(format(Messages.identifierIsNotAFunction, { identifier: this.callee.toString() }));
        }
        const cooked = this.quasi.quasis.map(x => x.cooked);
        Object.defineProperty(cooked, "raw", { value: this.quasi.quasis.map(x => x.raw) });
        return fn.apply(thisArg, [cooked, ...this.quasi.expressions.map(x => x.evaluate(scope))]);
    }
    toString() {
        return `${this.callee}${this.quasi}`;
    }
}
