import NodeType from "../node-type.js";
import TypeGuard from "../type-guard.js";
const updateFunctions = {
    "*++": (object, property) => object[property]++,
    "*--": (object, property) => object[property]--,
    "++*": (object, property) => ++object[property],
    "--*": (object, property) => --object[property],
};
export default class UpdateExpression {
    operation;
    _argument;
    _operator;
    _prefix;
    get argument() {
        return this._argument;
    }
    /* c8 ignore next 4 */
    set argument(value) {
        this._argument = value;
    }
    get operator() {
        return this._operator;
    }
    /* c8 ignore next 4 */
    set operator(value) {
        this._operator = value;
    }
    get prefix() {
        return this._prefix;
    }
    /* c8 ignore next 4 */
    set prefix(value) {
        this._prefix = value;
    }
    get type() {
        return NodeType.UpdateExpression;
    }
    constructor(argument, operator, prefix) {
        this._argument = argument;
        this._prefix = prefix;
        this._operator = operator;
        this.operation = this.prefix ? updateFunctions[`${this.operator}*`] : updateFunctions[`*${this.operator}`];
    }
    clone() {
        return new UpdateExpression(this.argument.clone(), this.operator, this.prefix);
    }
    evaluate(scope) {
        if (TypeGuard.isIdentifier(this.argument)) {
            return this.operation(scope, this.argument.name);
        }
        const object = this.argument.object.evaluate(scope);
        const property = TypeGuard.isIdentifier(this.argument.property) && !this.argument.computed ? this.argument.property.name : this.argument.property.evaluate(scope);
        return this.operation(object, property);
    }
    toString() {
        return this.prefix ? `${this.operator}${this.argument}` : `${this.argument}${this.operator}`;
    }
}
