import NodeType from "../node-type.js";
import { PATTERN } from "../symbols.js";
import TypeGuard from "../type-guard.js";
export default class ObjectPattern {
    _properties;
    [PATTERN];
    get properties() {
        return this._properties;
    }
    /* c8 ignore next 4 */
    set properties(value) {
        this._properties = value;
    }
    get type() {
        return NodeType.ObjectPattern;
    }
    constructor(properties) {
        this._properties = properties;
    }
    clone() {
        return new ObjectPattern(this.properties.map(x => x.clone()));
    }
    evaluate(scope, value) {
        const aliases = [];
        const currentScope = {};
        for (const property of this.properties) {
            if (TypeGuard.isAssignmentProperty(property)) {
                const key = TypeGuard.isIdentifier(property.key) && !property.computed ? property.key.name : `${property.key.evaluate(scope)}`;
                if (TypeGuard.isObjectPattern(property.value)) {
                    return property.value.evaluate(scope, value[key]);
                }
                else if (TypeGuard.isArrayPattern(property.value)) {
                    return property.value.evaluate(scope, value[key]);
                }
                const alias = `${TypeGuard.isAssignmentPattern(property.value) ? property.value.left.name : property.value.name}`;
                const aliasOrKey = property.shorthand ? alias : key;
                currentScope[alias] = TypeGuard.isAssignmentPattern(property.value)
                    ? property.value.right.evaluate(scope)
                    : currentScope[alias] = value[aliasOrKey];
                aliases.push(alias);
            }
            else {
                for (const alias of aliases) {
                    Reflect.deleteProperty(value, alias);
                }
                Object.assign(currentScope, property.evaluate(scope, value));
            }
        }
        return currentScope;
    }
    toString() {
        return this.properties.length > 0 ? `{ ${this.properties.map(x => x.toString()).join(", ")} }` : "{ }";
    }
}
